@@ -1,11 +1,11 @@
-# -*- coding: utf-8 -*-
+# -*- coding: ascii -*-
 """
 web2ldap.app.bulkmod: modify several entries found by prior search
 
 web2ldap - a web-based LDAP Client,
 see https://www.web2ldap.de for details
 
-(c) 1998-2021 by Michael Stroeder <michael@stroeder.com>
+(C) 1998-2022 by Michael Stroeder <michael@stroeder.com>
 
 This software is distributed under the terms of the
 Apache License Version 2.0 (Apache-2.0)
@@ -28,6 +28,46 @@ from .schema.syntaxes import syntax_registry, LDAPSyntaxValueError
 from .modify import modlist_ldif
 
 
+BULKMOD_FORM_TMPL = """
+{form_begin}
+{text_msg}
+<fieldset>
+  <legend>Search parameters</legend>
+  <table>
+    <tr>
+      <td>Search base:</td><td>{field_hidden_dn}</td>
+    </tr>
+    <tr>
+      <td>Search scope:</td><td>{field_hidden_scope}</td>
+    </tr>
+    <tr>
+      <td>Search filter:</td>
+      <td>
+        {field_hidden_filterstr}
+      </td>
+    </tr>
+  </table>
+</fieldset>
+<fieldset>
+  <legend>Bulk modify input</legend>
+  <p><input type="submit" name="bulkmod_submit" value="Next&gt;&gt;"></p>
+  <table>
+  <tr>
+    <td colspan="2">Superior DN:</td><td colspan="3">{field_bulkmod_newsuperior}</td>
+  </tr>
+  <tr>
+    <td colspan="2">Copy entries:</td><td colspan="3">{field_bulkmod_cp}</td>
+  </tr>
+  {input_fields}
+  </table>
+</fieldset>
+<fieldset>
+  <legend>Extended controls</legend>
+  {field_bulkmod_ctrl}
+</fieldset>
+</form>
+"""
+
 BULKMOD_CONFIRMATION_FORM_TMPL = """
 {form_begin}
 <p class="WarningMessage">
@@ -73,6 +113,81 @@ BULKMOD_CONFIRMATION_FORM_TMPL = """
 '</form>
 """
 
+BULKMOD_MESSAGE_TMPL = """
+<p class="SuccessMessage">Modified entries.</p>
+<table>
+  <tr>
+    <td>Modified entries:</td>
+    <td>{num_modify_success_count:d}</td>
+    <td>
+      <meter
+        min="0"
+        max="{num_all_modify_count:d}"
+        value="{num_modify_success_count:d}"
+        optimum="{num_all_modify_count:d}"
+        title="entries"
+      >
+        {num_modify_success_count:d}
+      </meter>
+    </td>
+  </tr>
+  <tr>
+    <td>Modification errors:</td>
+    <td>{num_modify_error_count:d}</td>
+    <td>
+      <meter
+        min="0"
+        max="{num_all_modify_count:d}"
+        value="{num_modify_error_count:d}"
+        optimum="0"
+        title="entries"
+      >
+        {num_modify_error_count:d}
+      </meter>
+    </td>
+  </tr>
+  <tr>
+    <td>Renamed/copied entries:</td>
+    <td>{num_modrdn_success_count:d}</td>
+    <td>
+      <meter
+        min="0"
+        max="{num_all_modrdn_count:d}"
+        value="{num_modrdn_success_count:d}"
+        optimum="{num_all_modrdn_count:d}"
+        title="entries"
+      >
+        {num_modrdn_success_count:d}
+      </meter>
+    </td>
+  </tr>
+  <tr>
+    <td>Rename/copy errors:</td>
+    <td>{num_modrdn_error_count:d}</td>
+    <td>
+      <meter
+        min="0"
+        max="{num_all_modrdn_count:d}"
+        value="{num_modrdn_error_count:d}"
+        optimum="0"
+        title="entries"
+      >
+        {num_modrdn_error_count:d}
+      </meter>
+    </td>
+  </tr>
+  <tr><td>Search base:</td><td>{text_dn}</td></tr>
+  <tr><td>Search scope:</td><td>{text_scope}</td></tr>
+  <tr><td>Time elapsed:</td><td>{num_elapsed_time:0.2f} seconds</td></tr>
+</table>
+{text_form_begin}
+{text_fields}
+  <p><input type="submit" name="bulkmod_submit" value="&lt;&lt;Back"></p>
+</form>
+{text_errors}
+{text_changes}
+"""
+
 
 def input_modlist(app, bulkmod_at, bulkmod_op, bulkmod_av):
 
@@ -90,7 +205,7 @@ def input_modlist(app, bulkmod_at, bulkmod_op, bulkmod_av):
             continue
 
         attr_instance = syntax_registry.get_at(
-            app, u'', app.schema, mod_type, None, entry=None,
+            app, '', app.schema, mod_type, None, entry=None,
         )
         try:
             mod_val = attr_instance.sanitize((bulkmod_av[i] or '').encode(app.ls.charset))
@@ -135,9 +250,9 @@ def bulkmod_input_form(
         input_errors,
     ):
     # Extend the input lists to at least one empty input row
-    bulkmod_at = bulkmod_at or [u'']
-    bulkmod_op = bulkmod_op or [u'']
-    bulkmod_av = bulkmod_av or [u'']
+    bulkmod_at = bulkmod_at or ['']
+    bulkmod_op = bulkmod_op or ['']
+    bulkmod_av = bulkmod_av or ['']
     error_attrs = sorted({bulkmod_at[i] for i in input_errors})
     if error_attrs:
         error_msg = '<p class="ErrorMessage">Invalid input: %s</p>' % (
@@ -156,12 +271,12 @@ def bulkmod_input_form(
         if len(bulkmod_at) < web2ldapcnf.max_searchparams:
             bulkmod_at.insert(insert_row_num+1, bulkmod_at[insert_row_num])
             bulkmod_op.insert(insert_row_num+1, bulkmod_op[insert_row_num])
-            bulkmod_av.insert(insert_row_num+1, u'')
+            bulkmod_av.insert(insert_row_num+1, '')
     # Generate a select field for the attribute type
     bulkmod_attr_select = attrtype_select_field(
         app,
         'bulkmod_at',
-        u'Attribute type',
+        'Attribute type',
         [], default_attr_options=None
     )
     # Output confirmation form
@@ -184,45 +299,7 @@ def bulkmod_input_form(
     ])
 
     app.outf.write(
-        """
-        {form_begin}
-        {text_msg}
-        <fieldset>
-          <legend>Search parameters</legend>
-          <table>
-            <tr>
-              <td>Search base:</td><td>{field_hidden_dn}</td>
-            </tr>
-            <tr>
-              <td>Search scope:</td><td>{field_hidden_scope}</td>
-            </tr>
-            <tr>
-              <td>Search filter:</td>
-              <td>
-                {field_hidden_filterstr}
-              </td>
-            </tr>
-          </table>
-        </fieldset>
-        <fieldset>
-          <legend>Bulk modify input</legend>
-          <p><input type="submit" name="bulkmod_submit" value="Next&gt;&gt;"></p>
-          <table>
-          <tr>
-            <td colspan="2">Superior DN:</td><td colspan="3">{field_bulkmod_newsuperior}</td>
-          </tr>
-          <tr>
-            <td colspan="2">Copy entries:</td><td colspan="3">{field_bulkmod_cp}</td>
-          </tr>
-          {input_fields}
-          </table>
-        </fieldset>
-        <fieldset>
-          <legend>Extended controls</legend>
-          {field_bulkmod_ctrl}
-        </fieldset>
-        </form>
-        """.format(
+        BULKMOD_FORM_TMPL.format(
             text_msg=error_msg,
             form_begin=app.begin_form('bulkmod', 'POST'),
             field_bulkmod_ctrl=app.form.field['bulkmod_ctrl'].input_html(default=app.form.field['bulkmod_ctrl'].value),
@@ -236,7 +313,7 @@ def bulkmod_input_form(
             ),
             field_bulkmod_newsuperior=app.form.field['bulkmod_newsuperior'].input_html(
                 default=bulkmod_newsuperior,
-                title=u'New superior DN where all entries are moved beneath',
+                title='New superior DN where all entries are moved beneath',
             ),
             field_bulkmod_cp=app.form.field['bulkmod_cp'].input_html(checked=bulkmod_cp),
         )
@@ -296,7 +373,7 @@ def bulkmod_confirmation_form(
                 bulkmod_newsuperior,
                 bulkmod_newsuperior
             ),
-            text_bulkmod_cp={False:u'Move', True:u'Copy'}[bulkmod_cp],
+            text_bulkmod_cp={False:'Move', True:'Copy'}[bulkmod_cp],
             num_entries=num_entries,
             num_referrals=num_referrals,
             text_ldifchangerecord=bulk_mod_list_ldif,
@@ -320,29 +397,29 @@ def w2l_bulkmod(app):
     bulkmod_op = app.form.getInputValue('bulkmod_op', [])
     bulkmod_av = app.form.getInputValue('bulkmod_av', [])
 
-    bulkmod_cp = app.form.getInputValue('bulkmod_cp', [u''])[0] == u'yes'
+    bulkmod_cp = app.form.getInputValue('bulkmod_cp', [''])[0] == 'yes'
 
     scope = int(app.form.getInputValue('scope', [str(app.ldap_url.scope or ldap0.SCOPE_BASE)])[0])
 
     bulkmod_filter = app.form.getInputValue(
         'filterstr',
         [(app.ldap_url.filterstr or '')]
-    )[0] or u'(objectClass=*)'
-    bulkmod_newsuperior = app.form.getInputValue('bulkmod_newsuperior', [u''])[0]
+    )[0] or '(objectClass=*)'
+    bulkmod_newsuperior = app.form.getInputValue('bulkmod_newsuperior', [''])[0]
 
     # Generate a list of requested LDAPv3 extended controls to be sent along
     # with the modify requests
     bulkmod_ctrl_oids = app.form.getInputValue('bulkmod_ctrl', [])
 
     if not len(bulkmod_at) == len(bulkmod_op) == len(bulkmod_av):
-        raise ErrorExit(u'Invalid bulk modification input.')
+        raise ErrorExit('Invalid bulk modification input.')
 
     bulk_mod_list, input_errors = input_modlist(
         app,
         bulkmod_at, bulkmod_op, bulkmod_av,
     )
 
-    if bulkmod_submit == u'Cancel':
+    if bulkmod_submit == 'Cancel':
 
         app.simple_message(
             'Canceled bulk modification.',
@@ -353,9 +430,9 @@ def w2l_bulkmod(app):
     elif not (bulk_mod_list or bulkmod_newsuperior) or \
          input_errors or \
          bulkmod_submit is None or \
-         bulkmod_submit == u'<<Back' or \
-         bulkmod_submit.startswith(u'+') or \
-         bulkmod_submit.startswith(u'-'):
+         bulkmod_submit == '<<Back' or \
+         bulkmod_submit.startswith('+') or \
+         bulkmod_submit.startswith('-'):
 
         bulkmod_input_form(
             app,
@@ -366,7 +443,7 @@ def w2l_bulkmod(app):
             input_errors
         )
 
-    elif bulkmod_submit == u'Next>>':
+    elif bulkmod_submit == 'Next>>':
 
         bulkmod_confirmation_form(
             app,
@@ -374,7 +451,7 @@ def w2l_bulkmod(app):
             bulkmod_newsuperior, bulk_mod_list, bulkmod_cp,
         )
 
-    elif bulkmod_submit == u'Apply':
+    elif bulkmod_submit == 'Apply':
 
         # now gather list of extended controls to be used with search request
         bulkmod_ctrl_oids = app.form.getInputValue('bulkmod_ctrl', [])
@@ -391,6 +468,8 @@ def w2l_bulkmod(app):
         ldap_error_html = []
 
         begin_time_stamp = time.time()
+        modify_success_count = modrdn_success_count = 0
+        modify_error_count = modrdn_error_count = 0
 
         # search the entries to be modified
         ldap_msgid = app.ls.l.search(
@@ -417,6 +496,7 @@ def w2l_bulkmod(app):
                     try:
                         app.ls.l.modify_s(rdat.dn_s, bulk_mod_list, req_ctrls=bulkmod_server_ctrls)
                     except ldap0.LDAPError as e:
+                        modify_error_count += 1
                         ldap_error_html.append(
                             '<dt>%s</dt><dd>%s</dd>' % (
                                 app.form.s2d(rdat.dn_s),
@@ -424,6 +504,7 @@ def w2l_bulkmod(app):
                             )
                         )
                     else:
+                        modify_success_count += 1
                         result_ldif_html.append(modlist_ldif(
                             rdat.dn_s, app.form, bulk_mod_list
                         ))
@@ -433,7 +514,7 @@ def w2l_bulkmod(app):
                     old_rdn = str(DNObj.from_str(rdat.dn_s).rdn())
                     try:
                         if bulkmod_cp:
-                            new_ldap_dn = u','.join((
+                            new_ldap_dn = ','.join((
                                 old_rdn,
                                 bulkmod_newsuperior,
                             ))
@@ -448,6 +529,7 @@ def w2l_bulkmod(app):
                                 delold=app.cfg_param('bulkmod_delold', 0),
                             )
                     except ldap0.LDAPError as e:
+                        modrdn_error_count += 1
                         ldap_error_html.append(
                             '<dt>%s</dt><dd>%s</dd>' % (
                                 app.form.s2d(rdat.dn_s),
@@ -455,6 +537,7 @@ def w2l_bulkmod(app):
                             )
                         )
                     else:
+                        modrdn_success_count += 1
                         result_ldif_html.append(
                             '<p>%s %s beneath %s</p>' % (
                                 {False:'Moved', True:'Copied'}[bulkmod_cp],
@@ -476,56 +559,31 @@ def w2l_bulkmod(app):
                 '\n'.join(result_ldif_html),
             )
 
-        num_mods = len(result_ldif_html)
-        num_errors = len(ldap_error_html)
-        num_sum = num_mods+num_errors
         app.simple_message(
             'Modified entries',
-            """
-            <p class="SuccessMessage">Modified entries.</p>
-            <table>
-              <tr>
-                <td>Modified entries:</td>
-                <td>%d</td>
-                <td>
-                  <meter min="0" max="%d" value="%d" optimum="%d" title="entries">%d</meter>
-                </td>
-              </tr>
-              <tr>
-                <td>Errors:</td>
-                <td>%d</td>
-                <td>
-                  <meter min="0" max="%d" value="%d" optimum="0" title="entries">%d</meter>
-                </td>
-              </tr>
-              <tr><td>Search base:</td><td>%s</td></tr>
-              <tr><td>Search scope:</td><td>%s</td></tr>
-              <tr><td>Time elapsed:</td><td>%0.2f seconds</td></tr>
-            </table>
-            %s
-            %s
-              <p><input type="submit" name="bulkmod_submit" value="&lt;&lt;Back"></p>
-            </form>
-            %s
-            %s
-            """ % (
-                num_mods,
-                num_sum, num_mods, num_sum, num_mods,
-                num_errors,
-                num_sum, num_errors, num_errors,
-                app.display_dn(app.dn),
-                ldap0.ldapurl.SEARCH_SCOPE_STR[scope],
-                end_time_stamp-begin_time_stamp,
-                app.begin_form('bulkmod', 'POST'),
-                app.form.hidden_input_html(ignored_fields=['bulkmod_submit']),
-                error_messages,
-                change_records,
+            BULKMOD_MESSAGE_TMPL.format(
+                # modify operations
+                num_modify_success_count=modify_success_count,
+                num_modify_error_count=modify_error_count,
+                num_all_modify_count=modify_success_count+modify_error_count,
+                # modrdn operations
+                num_modrdn_success_count=modrdn_success_count,
+                num_modrdn_error_count=modrdn_error_count,
+                num_all_modrdn_count=modrdn_success_count+modrdn_error_count,
+                # other stuff
+                text_dn=app.display_dn(app.dn),
+                text_scope=ldap0.ldapurl.SEARCH_SCOPE_STR[scope],
+                num_elapsed_time=end_time_stamp-begin_time_stamp,
+                text_form_begin=app.begin_form('bulkmod', 'POST'),
+                text_fields=app.form.hidden_input_html(ignored_fields=['bulkmod_submit']),
+                text_errors=error_messages,
+                text_changes=change_records,
             ),
             main_menu_list=main_menu(app),
         )
 
     else:
 
-        raise ErrorExit(u'Invalid bulk modification form data.')
+        raise ErrorExit('Invalid bulk modification form data.')
 
     # end of w2l_bulkmod()
