@@ -1,11 +1,11 @@
-# -*- coding: utf-8 -*-
+# -*- coding: ascii -*-
 """
 web2ldap.app.delete: delete one entry or several entries
 
 web2ldap - a web-based LDAP Client,
 see https://www.web2ldap.de for details
 
-(c) 1998-2021 by Michael Stroeder <michael@stroeder.com>
+(C) 1998-2022 by Michael Stroeder <michael@stroeder.com>
 
 This software is distributed under the terms of the
 Apache License Version 2.0 (Apache-2.0)
@@ -360,7 +360,7 @@ def w2l_delete(app):
     )
     delete_filter = app.form.getInputValue(
         'filterstr',
-        [app.ldap_url.filterstr or '(objectClass=*)']
+        [app.ldap_url.filterstr or None]
     )[0]
     delete_attr.sort()
     if delete_attr:
@@ -390,7 +390,7 @@ def w2l_delete(app):
         entry = ldap0.schema.models.Entry(app.schema, app.dn, ldap_res.entry_as)
         if delete_attr:
             inner_form = del_attr_form(app, entry, delete_attr)
-        elif delete_confirm == 'yes':
+        elif delete_filter is not None:
             inner_form = del_search_form(app, scope, delete_filter)
         else:
             inner_form = del_subtree_form(app, scope)
@@ -448,7 +448,7 @@ def w2l_delete(app):
             scope,
             delete_ctrl_tree_delete,
             delete_server_ctrls,
-            delete_filter,
+            delete_filter or '(objectClass=*)',
         )
         end_time_stamp = time.time()
 
