@@ -1,11 +1,11 @@
-# -*- coding: utf-8 -*-
+# -*- coding: ascii -*-
 """
 web2ldap.app.gui: basic functions for GUI elements
 
 web2ldap - a web-based LDAP Client,
 see https://www.web2ldap.de for details
 
-(c) 1998-2021 by Michael Stroeder <michael@stroeder.com>
+(C) 1998-2022 by Michael Stroeder <michael@stroeder.com>
 
 This software is distributed under the terms of the
 Apache License Version 2.0 (Apache-2.0)
@@ -38,6 +38,14 @@ from ..ldaputil import logdb_filter
 from .tmpl import read_template
 
 
+# Sequence of patterns to extract attribute names from LDAP diagnostic messages
+EXTRACT_INVALID_ATTR_PATTERNS = (
+    # OpenLDAP diagnostic messages
+    re.compile(r"(?P<at>[a-zA-Z0-9;-]+): value #(?P<index>[0-9]+) invalid per syntax"),
+    re.compile(r"object class '(?P<oc>[a-zA-Z0-9;-]+)' requires attribute '(?P<at>[a-zA-Z0-9;-]+)'"),
+)
+
+
 #---------------------------------------------------------------------------
 # Constants
 #---------------------------------------------------------------------------
@@ -66,43 +74,20 @@ def dn_anchor_hash(dn):
     return str(md5(dn.encode('utf-8')).hexdigest())
 
 
-def command_div(
-        commandlist,
-        div_id='CommandDiv',
-        separator=' ',
-        semantic_tag='nav',
-    ):
-    if semantic_tag:
-        start_tag = '<%s>' % semantic_tag
-        end_tag = '<%s>' % semantic_tag
-    else:
-        start_tag = ''
-        end_tag = ''
-    if commandlist:
-        return '%s<p id="%s" class="CT">\n%s\n</p>%s\n' % (
-            start_tag,
-            div_id,
-            (separator).join(commandlist),
-            end_tag,
-        )
-    return ''
-    # end of command_div()
-
-
 def simple_main_menu(app):
-    main_menu = [app.anchor('', 'Connect', [])]
+    mil = [app.anchor('', 'Connect', [])]
     if app.check_access('monitor'):
-        main_menu.append(app.anchor('monitor', 'Monitor', []))
+        mil.append(app.anchor('monitor', 'Monitor', []))
     if DNS_AVAIL and app.check_access('locate'):
-        main_menu.append(app.anchor('locate', 'DNS lookup', []))
-    return main_menu
+        mil.append(app.anchor('locate', 'DNS lookup', []))
+    return mil
 
 
 def context_menu_single_entry(app, vcard_link=0, dds_link=0, entry_uuid=None):
     """
     Output the context menu for a single entry
     """
-    dn_disp = app.dn or u'Root DSE'
+    dn_disp = app.dn or 'Root DSE'
     mil = [
         app.anchor(
             'read', 'Raw',
@@ -111,7 +96,7 @@ def context_menu_single_entry(app, vcard_link=0, dds_link=0, entry_uuid=None):
                 ('read_output', 'table'),
                 ('read_expandattr', '*')
             ],
-            title=u'Display entry\r\n%s\r\nas raw attribute type/value list' % (dn_disp)
+            title='Display entry\r\n%s\r\nas raw attribute type/value list' % (dn_disp)
         ),
     ]
     if app.dn:
@@ -124,41 +109,41 @@ def context_menu_single_entry(app, vcard_link=0, dds_link=0, entry_uuid=None):
                     ('dn', app.dn),
                     ('login_who', app.dn),
                 ],
-                title=u'Connect and bind new session as\r\n%s' % (app.dn)
+                title='Connect and bind new session as\r\n%s' % (app.dn)
             ),
             app.anchor(
                 'modify', 'Modify',
                 [('dn', app.dn)],
-                title=u'Modify entry\r\n%s' % (app.dn)
+                title='Modify entry\r\n%s' % (app.dn)
             ),
             app.anchor(
                 'rename', 'Rename',
                 [('dn', app.dn)],
-                title=u'Rename/move entry\r\n%s' % (app.dn)
+                title='Rename/move entry\r\n%s' % (app.dn)
             ),
             app.anchor(
                 'delete', 'Delete',
                 [('dn', app.dn)],
-                title=u'Delete entry and/or subtree\r\n%s' % (app.dn)
+                title='Delete entry and/or subtree\r\n%s' % (app.dn)
             ),
             app.anchor(
                 'passwd', 'Password',
                 [('dn', app.dn), ('passwd_who', app.dn)],
-                title=u'Set password for entry\r\n%s' % (app.dn)
+                title='Set password for entry\r\n%s' % (app.dn)
             ),
             app.anchor(
                 'groupadm', 'Groups',
                 [('dn', app.dn)],
-                title=u'Change group membership of entry\r\n%s' % (app.dn)
+                title='Change group membership of entry\r\n%s' % (app.dn)
             ),
             app.anchor(
                 'add', 'Clone',
                 [
                     ('dn', app.parent_dn),
                     ('add_clonedn', app.dn),
-                    ('in_ft', u'Template'),
+                    ('in_ft', 'Template'),
                 ],
-                title=u'Clone entry\r\n%s\r\nbeneath %s' % (app.dn, app.parent_dn)
+                title='Clone entry\r\n%s\r\nbeneath %s' % (app.dn, app.parent_dn)
             ),
         ])
 
@@ -167,7 +152,7 @@ def context_menu_single_entry(app, vcard_link=0, dds_link=0, entry_uuid=None):
             app.anchor(
                 'read', 'vCard',
                 [('dn', app.dn), ('read_output', 'vcard')],
-                title=u'Export entry\r\n%s\r\nas vCard' % (dn_disp)
+                title='Export entry\r\n%s\r\nas vCard' % (dn_disp)
             )
         )
 
@@ -176,13 +161,13 @@ def context_menu_single_entry(app, vcard_link=0, dds_link=0, entry_uuid=None):
             app.anchor(
                 'dds', 'Refresh',
                 [('dn', app.dn)],
-                title=u'Refresh dynamic entry %s' % (dn_disp)
+                title='Refresh dynamic entry %s' % (dn_disp)
             )
         )
 
     if app.audit_context:
-        accesslog_any_filterstr = logdb_filter(u'auditObject', app.dn, entry_uuid)
-        accesslog_write_filterstr = logdb_filter(u'auditWriteObject', app.dn, entry_uuid)
+        accesslog_any_filterstr = logdb_filter('auditObject', app.dn, entry_uuid)
+        accesslog_write_filterstr = logdb_filter('auditWriteObject', app.dn, entry_uuid)
         mil.extend([
             app.anchor(
                 'search', 'Audit access',
@@ -191,7 +176,7 @@ def context_menu_single_entry(app, vcard_link=0, dds_link=0, entry_uuid=None):
                     ('filterstr', accesslog_any_filterstr),
                     ('scope', str(ldap0.SCOPE_ONELEVEL)),
                 ],
-                title=u'Complete audit trail for entry\r\n%s' % (app.dn),
+                title='Complete audit trail for entry\r\n%s' % (app.dn),
             ),
             app.anchor(
                 'search', 'Audit writes',
@@ -200,7 +185,7 @@ def context_menu_single_entry(app, vcard_link=0, dds_link=0, entry_uuid=None):
                     ('filterstr', accesslog_write_filterstr),
                     ('scope', str(ldap0.SCOPE_ONELEVEL)),
                 ],
-                title=u'Audit trail of write access to entry\r\n%s' % (app.dn),
+                title='Audit trail of write access to entry\r\n%s' % (app.dn),
             ),
         ])
 
@@ -209,7 +194,7 @@ def context_menu_single_entry(app, vcard_link=0, dds_link=0, entry_uuid=None):
     except KeyError:
         pass
     else:
-        changelog_filterstr = logdb_filter(u'changeLogEntry', app.dn, entry_uuid)
+        changelog_filterstr = logdb_filter('changeLogEntry', app.dn, entry_uuid)
         mil.append(
             app.anchor(
                 'search', 'Change log',
@@ -218,7 +203,7 @@ def context_menu_single_entry(app, vcard_link=0, dds_link=0, entry_uuid=None):
                     ('filterstr', changelog_filterstr),
                     ('scope', str(ldap0.SCOPE_ONELEVEL)),
                 ],
-                title=u'Audit trail of write access to current entry',
+                title='Audit trail of write access to current entry',
             )
         )
 
@@ -239,7 +224,7 @@ def context_menu_single_entry(app, vcard_link=0, dds_link=0, entry_uuid=None):
                 ),
                 ('scope', str(ldap0.SCOPE_SUBTREE)),
             ],
-            title=u'Find connections of this user in monitor database',
+            title='Find connections of this user in monitor database',
         ))
 
     return mil
@@ -261,12 +246,12 @@ def main_menu(app):
                     (
                         ('dn', app.parent_dn),
                         ('scope', str(ldap0.SCOPE_ONELEVEL)),
-                        ('searchform_mode', u'adv'),
-                        ('search_attr', u'objectClass'),
+                        ('searchform_mode', 'adv'),
+                        ('search_attr', 'objectClass'),
                         ('search_option', '({at}=*)'),
                         ('search_string', ''),
                     ),
-                    title=u'List direct subordinates of %s' % (app.parent_dn or u'Root DSE'),
+                    title='List direct subordinates of %s' % (app.parent_dn or 'Root DSE'),
                 )
             )
 
@@ -276,17 +261,17 @@ def main_menu(app):
                 (
                     ('dn', app.dn),
                     ('scope', str(ldap0.SCOPE_ONELEVEL)),
-                    ('searchform_mode', u'adv'),
-                    ('search_attr', u'objectClass'),
+                    ('searchform_mode', 'adv'),
+                    ('search_attr', 'objectClass'),
                     ('search_option', '({at}=*)'),
-                    ('search_string', u''),
+                    ('search_string', ''),
                 ),
-                title=u'List direct subordinates of %s' % (app.dn or u'Root DSE'),
+                title='List direct subordinates of %s' % (app.dn or 'Root DSE'),
             ),
             app.anchor(
                 'searchform', 'Search',
                 (('dn', app.dn),),
-                title=u'Enter search criteria in input form',
+                title='Enter search criteria in input form',
             ),
         ))
 
@@ -294,7 +279,7 @@ def main_menu(app):
             app.anchor(
                 'dit', 'Tree',
                 [('dn', app.dn)],
-                title=u'Display tree around %s' % (app.dn or u'Root DSE'),
+                title='Display tree around %s' % (app.dn or 'Root DSE'),
                 anchor_id=dn_anchor_hash(app.dn_obj)
             ),
         )
@@ -302,8 +287,8 @@ def main_menu(app):
         mil.append(
             app.anchor(
                 'read', 'Read',
-                [('dn', app.dn), ('read_nocache', u'1')],
-                title=u'Display entry %s' % (app.dn or u'Root DSE'),
+                [('dn', app.dn), ('read_nocache', '1')],
+                title='Display entry %s' % (app.dn or 'Root DSE'),
             ),
         )
 
@@ -311,27 +296,27 @@ def main_menu(app):
             app.anchor(
                 'add', 'New entry',
                 [('dn', app.dn)],
-                title=u'Add a new entry below of %s' % (app.dn or u'Root DSE')
+                title='Add a new entry below of %s' % (app.dn or 'Root DSE')
             ),
             app.anchor(
                 'conninfo', 'ConnInfo',
                 [('dn', app.dn)],
-                title=u'Show information about HTTP and LDAP connections'
+                title='Show information about HTTP and LDAP connections'
             ),
             app.anchor(
                 'params', 'Params',
                 [('dn', app.dn)],
-                title=u'Tweak parameters used for LDAP operations (controls etc.)'
+                title='Tweak parameters used for LDAP operations (controls etc.)'
             ),
-            app.anchor('login', 'Bind', [('dn', app.dn)], title=u'Login to directory'),
-            app.anchor('oid', 'Schema', [('dn', app.dn)], title=u'Browse/view subschema'),
+            app.anchor('login', 'Bind', [('dn', app.dn)], title='Login to directory'),
+            app.anchor('oid', 'Schema', [('dn', app.dn)], title='Browse/view subschema'),
         ))
 
-        mil.append(app.anchor('disconnect', 'Disconnect', (), title=u'Disconnect from LDAP server'))
+        mil.append(app.anchor('disconnect', 'Disconnect', (), title='Disconnect from LDAP server'))
 
     else:
 
-        mil.append(app.anchor('', 'Connect', (), title=u'New connection to LDAP server'))
+        mil.append(app.anchor('', 'Connect', (), title='New connection to LDAP server'))
 
     return mil
     # end of main_menu()
@@ -343,7 +328,7 @@ def dit_navigation(app):
             'read',
             app.form.s2d(str(app.dn_obj.slice(i, i+1)) or '[Root DSE]'),
             [('dn', str(app.dn_obj.slice(i, None)))],
-            title=u'Jump to %s' % (str(app.dn_obj.slice(i, None))),
+            title='Jump to %s' % (str(app.dn_obj.slice(i, None))),
         )
         for i in range(len(app.dn_obj))
     ]
@@ -351,7 +336,7 @@ def dit_navigation(app):
         app.anchor(
             'read', '[Root DSE]',
             [('dn', '')],
-            title=u'Jump to root DSE',
+            title='Jump to root DSE',
         )
     )
     return dnil
@@ -370,7 +355,7 @@ def top_section(
     header(app, 'text/html', app.form.accept_charset)
 
     # Read the template file for TopSection
-    top_template_str = read_template(app, 'top_template', u'top section')
+    top_template_str = read_template(app, 'top_template', 'top section')
 
     script_name = escape_html(app.form.script_name)
 
@@ -384,23 +369,17 @@ def top_section(
         'web2ldap_version': escape_html(__version__),
         'command': app.command,
         'ldap_url': '',
-        'ldap_uri': '-/-',
-        'description': '',
-        'who': '-/-',
-        'dn': '-/-',
-        'dit_navi': '-/-',
-        'main_menu': command_div(
-            main_menu_list,
-            div_id='MainMenu',
-            separator='\n',
-            semantic_tag=None,
-        ),
-        'context_menu': command_div(
-            context_menu_list,
-            div_id='ContextMenu',
-            separator='\n',
-            semantic_tag=None,
-        ),
+        'ldap_uri': '',
+        'description': 'Disconnected',
+        'who': '',
+        'dn': '',
+        'dit_navi': '',
+        'main_menu': '\n'.join(main_menu_list),
+        'context_menu': (
+            '\n'.join(context_menu_list)
+            if context_menu_list
+            else '\n'
+        )
     }
     template_dict.update([(k, escape_html(str(v))) for k, v in app.env.items()])
 
@@ -409,7 +388,7 @@ def top_section(
         template_dict.update({
             'ldap_url': app.ls.ldap_url(app.dn),
             'ldap_uri': app.form.s2d(app.ls.uri),
-            'description': escape_html(app.cfg_param('description', u'')),
+            'description': escape_html(app.cfg_param('description', '')),
             'dit_navi': ',\n'.join(dit_navigation(app)),
             'dn': app.form.s2d(app.dn),
         })
@@ -453,6 +432,7 @@ def attrtype_select_field(
     attr_select = SelectField(
         field_name, field_desc, 1,
         options=sorted_attr_options,
+        auto_add_option=True,
     )
     attr_select.charset = app.form.accept_charset
     return attr_select
@@ -485,7 +465,7 @@ def header(app, content_type, charset, more_headers=None):
     app.outf.reset()
     if app.form.next_cookie:
         for _, cookie in app.form.next_cookie.items():
-            headers.append(('Set-Cookie', str(cookie)[12:]))
+            headers.append(('Set-Cookie', cookie.output()[12:]))
     if app.form.env.get('HTTPS', 'off') == 'on' and \
        'Strict-Transport-Security' not in web2ldapcnf.http_headers:
         headers.append(('Strict-Transport-Security', 'max-age=15768000 ; includeSubDomains'))
@@ -501,7 +481,7 @@ def footer(app):
 def search_root_field(
         app,
         name='dn',
-        text=u'Search Root',
+        text='Search Root',
         default=None,
         search_root_searchurl=None,
     ):
@@ -547,7 +527,7 @@ def search_root_field(
     if '' in dn_select_list:
         dn_select_list.remove('')
     # Add root search base string with description
-    dn_select_list.add((u'', u'- World -'))
+    dn_select_list.add(('', '- World -'))
     srf = SelectField(
         name, text, 1,
         size=1,
@@ -576,24 +556,29 @@ def invalid_syntax_message(app, invalid_attrs):
     )
 
 
-def extract_invalid_syntax(app, ldap_err):
+def extract_invalid_attr(app, ldap_err):
     """
-    try to extract error message and invalid attributes dict from LDAPError instance
+    Extract invalid attributes dict from diagnostic message in LDAPError instance
     """
     try:
         # try to extract OpenLDAP-specific info message
         info = ldap_err.args[0].get('info', b'').decode(app.ls.charset)
-        invalid_attr, invalid_index = re.match(
-            '([a-zA-Z0-9;-]+): value #([0-9]+) invalid per syntax',
-            info,
-        ).groups()
     except (AttributeError, IndexError, UnicodeDecodeError):
         # could not extract useful info
-        invalid_attrs = {}
-        error_msg = app.ldap_error_msg(ldap_err)
-    else:
-        invalid_attrs = {invalid_attr: [int(invalid_index)]}
-        error_msg = invalid_syntax_message(app, invalid_attrs)
+        return (app.ldap_error_msg(ldap_err), {})
+    invalid_attrs = {}
+    for extract_pattern in EXTRACT_INVALID_ATTR_PATTERNS:
+        match = extract_pattern.match(info)
+        if match is None:
+            continue
+        matches = match.groupdict()
+        if 'at' in matches:
+            invalid_attrs[matches['at']] = [int(matches.get('index', 0))]
+            if isinstance(ldap_err, ldap0.INVALID_SYNTAX):
+                error_msg = invalid_syntax_message(app, invalid_attrs)
+            else:
+                error_msg = app.ldap_error_msg(ldap_err)
+            break
     return error_msg, invalid_attrs
 
 
