@@ -1,11 +1,11 @@
-# -*- coding: utf-8 -*-
+# -*- coding: ascii -*-
 """
 web2ldap.app.passwd: change password associated with entry
 
 web2ldap - a web-based LDAP Client,
 see https://www.web2ldap.de for details
 
-(c) 1998-2021 by Michael Stroeder <michael@stroeder.com>
+(C) 1998-2022 by Michael Stroeder <michael@stroeder.com>
 
 This software is distributed under the terms of the
 Apache License Version 2.0 (Apache-2.0)
@@ -397,7 +397,7 @@ def w2l_passwd(app):
             new_passwd_value = unicode_pwd(password=passwd_input.encode('utf-8'))
             passwd_scheme = ''
             if old_password:
-                old_passwd_value = unicode_pwd(old_password)
+                old_passwd_value = unicode_pwd(old_password.encode('utf-8'))
         else:
             # Assume standard password attribute userPassword
             passwd_attr_type = b'userPassword'
@@ -476,7 +476,6 @@ def w2l_passwd(app):
 
         # Modify password
         try:
-            print('passwd_modlist', repr(passwd_modlist))
             app.ls.modify(passwd_who, passwd_modlist)
         except (
                 ldap0.CONSTRAINT_VIOLATION,
