@@ -1,11 +1,11 @@
-# -*- coding: utf-8 -*-
+# -*- coding: ascii -*-
 """
 web2ldap.app.schema.syntaxes: classes for known attribute types
 
 web2ldap - a web-based LDAP Client,
 see https://www.web2ldap.de for details
 
-(c) 1998-2021 by Michael Stroeder <michael@stroeder.com>
+(C) 1998-2022 by Michael Stroeder <michael@stroeder.com>
 
 This software is distributed under the terms of the
 Apache License Version 2.0 (Apache-2.0)
@@ -35,6 +35,8 @@ from typing import (
     Tuple,
 )
 
+import iso3166
+
 try:
     import defusedxml.ElementTree
 except ImportError:
@@ -42,6 +44,13 @@ except ImportError:
 else:
     DEFUSEDXML_AVAIL = True
 
+try:
+    import phonenumbers
+except ImportError:
+    PHONENUMBERS_AVAIL = False
+else:
+    PHONENUMBERS_AVAIL = True
+
 from collections import defaultdict
 from io import BytesIO
 
@@ -491,11 +500,11 @@ class Audio(Binary):
         mimetype = self.mime_type
         return (
             '<embed type="%s" autostart="false" '
-            'src="%s/read/%s?dn=%s&amp;read_attr=%s&amp;read_attrindex=%d">'
+            'src="%s?dn=%s&amp;read_attr=%s&amp;read_attrindex=%d">'
             '%d bytes of audio data (%s)'
         ) % (
             mimetype,
-            self._app.form.script_name, self._app.sid,
+            self._app.form.action_url('read', self._app.sid),
             urllib.parse.quote(self._dn.encode(self._app.form.accept_charset)),
             urllib.parse.quote(self._at),
             vidx,
@@ -507,7 +516,7 @@ class Audio(Binary):
 class DirectoryString(LDAPSyntax):
     """
     Plugin class for LDAP syntax 'Directory String'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.6)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.6)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.15'
     desc: str = 'Directory String'
@@ -529,7 +538,7 @@ class DirectoryString(LDAPSyntax):
 class DistinguishedName(DirectoryString):
     """
     Plugin class for LDAP syntax 'DN'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.9)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.9)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.12'
     desc: str = 'Distinguished Name'
@@ -580,6 +589,8 @@ class DistinguishedName(DirectoryString):
                 ref_oc = None
                 ref_attr, ref_text, ref_dn, ref_title = ref_attr_tuple
             ref_attr = ref_attr or self._at
+            if ref_attr not in self._schema.name2oid[AttributeType]:
+                continue
             ref_dn = ref_dn or self._dn
             ref_title = ref_title or 'Search %s entries referencing entry %s in attribute %s' % (
                 ref_oc, self.av_u, ref_attr,
@@ -647,7 +658,7 @@ class AuthzDN(DistinguishedName):
 class NameAndOptionalUID(DistinguishedName):
     """
     Plugin class for LDAP syntax 'Name and Optional UID'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.21)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.21)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.34'
     desc: str = 'Name And Optional UID'
@@ -685,7 +696,7 @@ class NameAndOptionalUID(DistinguishedName):
 class BitString(DirectoryString):
     """
     Plugin class for LDAP syntax 'Bit String'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.2)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.2)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.6'
     desc: str = 'Bit String'
@@ -695,7 +706,7 @@ class BitString(DirectoryString):
 class IA5String(DirectoryString):
     """
     Plugin class for LDAP syntax 'IA5 String'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.15)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.15)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.26'
     desc: str = 'IA5 String'
@@ -711,7 +722,7 @@ class IA5String(DirectoryString):
 class GeneralizedTime(IA5String):
     """
     Plugin class for LDAP syntax 'Generalized Time'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.13)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.13)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.24'
     desc: str = 'Generalized Time'
@@ -884,7 +895,7 @@ class NotAfter(GeneralizedTime):
 class UTCTime(GeneralizedTime):
     """
     Plugin class for LDAP syntax 'UTC Time'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.34)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.34)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.53'
     desc: str = 'UTC Time'
@@ -915,7 +926,7 @@ class NullTerminatedDirectoryString(DirectoryString):
 class OtherMailbox(DirectoryString):
     """
     Plugin class for LDAP syntax 'Other Mailbox'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.27)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.27)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.39'
     desc: str = 'Other Mailbox'
@@ -925,7 +936,7 @@ class OtherMailbox(DirectoryString):
 class Integer(IA5String):
     """
     Plugin class for LDAP syntax 'Integer'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.16)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.16)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.27'
     desc: str = 'Integer'
@@ -1110,9 +1121,8 @@ class Uri(DirectoryString):
             return '<code>%s</code>' % (
                 DirectoryString.display(self, vidx=False, links=False)
             )
-        return '<a href="%s/urlredirect/%s?%s">%s%s</a>' % (
-            self._app.form.script_name,
-            self._app.sid,
+        return '<a href="%s?%s">%s%s</a>' % (
+            self._app.form.action_url('urlredirect', self._app.sid),
             self._app.form.s2d(url),
             self._app.form.s2d(label),
             self._app.form.s2d(display_url),
@@ -1175,11 +1185,8 @@ class Image(Binary):
                 else:
                     size_attr_html = 'width="%d" height="%d"' % (width, height)
         attr_value_len = len(self._av)
-        img_link = (
-            '%s/read/%s'
-            '?dn=%s&amp;read_attr=%s&amp;read_attrindex=%d'
-        ) % (
-            self._app.form.script_name, self._app.sid,
+        img_link = '%s?dn=%s&amp;read_attr=%s&amp;read_attrindex=%d' % (
+            self._app.form.action_url('read', self._app.sid),
             urllib.parse.quote(self._dn),
             urllib.parse.quote(self._at),
             vidx,
@@ -1207,7 +1214,7 @@ class Image(Binary):
 class JPEGImage(Image):
     """
     Plugin class for LDAP syntax 'JPEG'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.17)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.17)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.28'
     desc: str = 'JPEG image'
@@ -1219,7 +1226,7 @@ class JPEGImage(Image):
 class PhotoG3Fax(Binary):
     """
     Plugin class for LDAP syntax 'Fax'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.12)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.12)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.23'
     desc: str = 'Photo (G3 fax)'
@@ -1230,7 +1237,7 @@ class PhotoG3Fax(Binary):
 class OID(IA5String):
     """
     Plugin class for LDAP syntax 'OID'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.26)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.26)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.38'
     desc: str = 'OID'
@@ -1274,7 +1281,7 @@ class OID(IA5String):
                     self.av_u,
                     AttributeType,
                     name_template='{name}\n{anchor}',
-                    link_text='&raquo',
+                    link_text='&raquo;',
                 )
             if self._at.lower() == 'structuralobjectclass':
                 name_template = '{name}\n{anchor}'
@@ -1286,7 +1293,7 @@ class OID(IA5String):
                 self.av_u,
                 ObjectClass,
                 name_template=name_template,
-                link_text='&raquo',
+                link_text='&raquo;',
             )
         return '<strong>%s</strong> (%s):<br>%s (see %s)' % (
             self._app.form.s2d(name),
@@ -1328,7 +1335,7 @@ class LDAPUrl(Uri):
 class OctetString(Binary):
     """
     Plugin class for LDAP syntax 'Octet String'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.25)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.25)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.40'
     desc: str = 'Octet String'
@@ -1459,7 +1466,7 @@ class PreformattedMultilineText(MultilineText):
 class PostalAddress(MultilineText):
     """
     Plugin class for LDAP syntax 'Postal Address'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.28)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.28)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.41'
     desc: str = 'Postal Address'
@@ -1479,7 +1486,7 @@ class PostalAddress(MultilineText):
 class PrintableString(DirectoryString):
     """
     Plugin class for LDAP syntax 'Printable String'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.29)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.29)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.44'
     desc: str = 'Printable String'
@@ -1490,7 +1497,7 @@ class PrintableString(DirectoryString):
 class NumericString(PrintableString):
     """
     Plugin class for LDAP syntax 'Numeric String'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.23)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.23)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.36'
     desc: str = 'Numeric String'
@@ -1500,7 +1507,7 @@ class NumericString(PrintableString):
 class EnhancedGuide(PrintableString):
     """
     Plugin class for LDAP syntax 'Enhanced Guide'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.10)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.10)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.21'
     desc: str = 'Enhanced Search Guide'
@@ -1509,7 +1516,7 @@ class EnhancedGuide(PrintableString):
 class Guide(EnhancedGuide):
     """
     Plugin class for LDAP syntax 'Search Guide'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.14)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.14)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.25'
     desc: str = 'Search Guide'
@@ -1518,17 +1525,41 @@ class Guide(EnhancedGuide):
 class TelephoneNumber(PrintableString):
     """
     Plugin class for LDAP syntax ''
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.31)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.31)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.50'
     desc: str = 'Telephone Number'
     pattern = re.compile('^[0-9+x(). /-]+$')
 
+    def sanitize(self, attr_value: bytes) -> bytes:
+        if PHONENUMBERS_AVAIL:
+            try:
+                attr_value = phonenumbers.format_number(
+                    phonenumbers.parse(
+                        attr_value.decode('ascii'),
+                        region=(
+                            self._entry['c'][0].decode('ascii')
+                            if 'c' in self._entry
+                            else None
+                        ),
+                    ),
+                    phonenumbers.PhoneNumberFormat.INTERNATIONAL,
+                ).encode('ascii')
+            except (
+                    UnicodeDecodeError,
+                    ValueError,
+                    phonenumbers.phonenumberutil.NumberParseException,
+                ):
+                attr_value = PrintableString.sanitize(self, attr_value)
+        else:
+            attr_value = PrintableString.sanitize(self, attr_value)
+        return attr_value
+
 
 class FacsimileTelephoneNumber(TelephoneNumber):
     """
     Plugin class for LDAP syntax 'Facsimile Telephone Number'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.11)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.11)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.22'
     desc: str = 'Facsimile Number'
@@ -1543,7 +1574,7 @@ class FacsimileTelephoneNumber(TelephoneNumber):
 class TelexNumber(PrintableString):
     """
     Plugin class for LDAP syntax 'Telex Number'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.33)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.33)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.52'
     desc: str = 'Telex Number'
@@ -1553,7 +1584,7 @@ class TelexNumber(PrintableString):
 class TeletexTerminalIdentifier(PrintableString):
     """
     Plugin class for LDAP syntax 'Teletex Terminal Identifier'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.32)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.32)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.51'
     desc: str = 'Teletex Terminal Identifier'
@@ -2168,7 +2199,7 @@ class DerefDynamicDNSelectList(DynamicDNSelectList):
 class Boolean(SelectList, IA5String):
     """
     Plugin class for LDAP syntax 'Boolean'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.3)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.3)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.7'
     desc: str = 'Boolean'
@@ -2194,25 +2225,30 @@ class Boolean(SelectList, IA5String):
         return IA5String.display(self, vidx, links)
 
 
-class CountryString(PropertiesSelectList):
+class CountryString(SelectList):
     """
     Plugin class for LDAP syntax 'Country String'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.4)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.4)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.11'
     desc: str = 'Two letter country string as listed in ISO 3166-2'
-    properties_pathname = os.path.join(
-        ETC_DIR, 'properties', 'attribute_select_c.properties'
-    )
     sani_funcs = (
         bytes.strip,
     )
 
+    def get_attr_value_dict(self) -> Dict[str, str]:
+        # Enable empty value in any case
+        attr_value_dict: Dict[str, str] = {'': '-/-'}
+        attr_value_dict.update({
+            alpha2: cty.name  for alpha2, cty in iso3166.countries_by_alpha2.items()
+        })
+        return attr_value_dict
+
 
 class DeliveryMethod(PrintableString):
     """
     Plugin class for LDAP syntax 'Delivery Method'
-    (see https://tools.ietf.org/html/rfc4517#section-3.3.5)
+    (see https://datatracker.ietf.org/doc/html/rfc4517#section-3.3.5)
     """
     oid: str = '1.3.6.1.4.1.1466.115.121.1.14'
     desc: str = 'Delivery Method'
@@ -2341,7 +2377,7 @@ class DNSDomain(IA5String):
     oid: str = 'DNSDomain-oid'
     desc: str = 'DNS domain name (see RFC 1035)'
     pattern = re.compile(r'^(\*|[a-zA-Z0-9_-]+)(\.[a-zA-Z0-9_-]+)*$')
-    # see https://tools.ietf.org/html/rfc2181#section-11
+    # see https://datatracker.ietf.org/doc/html/rfc2181#section-11
     max_len: int = min(255, IA5String.max_len)
     sani_funcs = (
         bytes.lower,
@@ -2414,7 +2450,7 @@ class RFC822Address(DNSDomain, IA5String):
 class DomainComponent(DNSDomain):
     """
     Plugin class for a single DNS label
-    (see https://tools.ietf.org/html/rfc2181#section-11)
+    (see https://datatracker.ietf.org/doc/html/rfc2181#section-11)
     """
     oid: str = 'DomainComponent-oid'
     desc: str = 'DNS domain name component'
@@ -2666,7 +2702,7 @@ class SchemaDescription(DirectoryString):
         if self.schema_cls is None:
             return DirectoryString._validate(self, attr_value)
         try:
-            se_obj = self.schema_cls(self._app.ls.uc_decode(attr_value)[0])
+            _ = self.schema_cls(self._app.ls.uc_decode(attr_value)[0])
         except (IndexError, ValueError):
             return False
         return True
