@@ -1,11 +1,11 @@
-# -*- coding: utf-8 -*-
+# -*- coding: ascii -*-
 """
 ldaputil.ldapurl - extended LDAPUrl class
 
 web2ldap - a web-based LDAP Client,
 see https://www.web2ldap.de for details
 
-(c) 1998-2021 by Michael Stroeder <michael@stroeder.com>
+(C) 1998-2022 by Michael Stroeder <michael@stroeder.com>
 
 This software is distributed under the terms of the
 Apache License Version 2.0 (Apache-2.0)
@@ -36,7 +36,7 @@ class ExtendedLDAPUrl(ldap0.ldapurl.LDAPUrl):
             try:
                 result = int(self.start_tls or '0')
             except ValueError:
-                raise ValueError(u'LDAP URL extension x-starttls must be integer 0, 1 or 2.')
+                raise ValueError('LDAP URL extension x-starttls must be integer 0, 1 or 2.')
         else:
             result = int(ext_starttls.critical) + int(ext_starttls.extype.lower() == 'starttls')
         return max(result, min_starttls) # get_starttls_extop()
