@@ -1,4 +1,4 @@
-# -*- coding: utf-8 -*-
+# -*- coding: ascii -*-
 """
 ldapoidreg - Simple dictionary registry for LDAP-related OIDs.
 
@@ -8,7 +8,7 @@ LDAP-related OIDs (e.g. in RootDSE)
 web2ldap - a web-based LDAP Client,
 see http://www.web2ldap.de for details
 
-(c) 1998-2021 by Michael Stroeder <michael@stroeder.com>
+(C) 1998-2022 by Michael Stroeder <michael@stroeder.com>
 
 Comprehensive list initially contributed by Norbert Klasen
 """
@@ -2215,7 +2215,7 @@ OID_LIST = (
         "IBM Directory Server"),
     (
         "1.3.18.0.2.32.84",
-        "Reread the DB2® Password",
+        "Reread the DB2 Password",
         "Server re-reads the DB2 password to identify any change in DB2 password specified in configuration.",
         "IBM Directory Server"),
     (
