@@ -1,8 +1,8 @@
-# -*- coding: utf-8 -*-
+# -*- coding: ascii -*-
 """
 web2ldap.web.forms - class library for handling <FORM> input
 
-(c) 1998-2021 by Michael Stroeder <michael@stroeder.com>
+(C) 1998-2022 by Michael Stroeder <michael@stroeder.com>
 
 This software is distributed under the terms of the
 Apache License Version 2.0 (Apache-2.0)
@@ -510,6 +510,7 @@ class Select(Radio):
             size=1,
             ignoreCase=0,
             multiSelect=0,
+            auto_add_option=False,
         ):
         """
         Additional parameters:
@@ -525,6 +526,7 @@ class Select(Radio):
         self.size = size
         self.multiSelect = multiSelect
         self.ignoreCase = ignoreCase
+        self.auto_add_option = auto_add_option
         Radio.__init__(self, name, text, maxValues, required, default, accesskey, options)
 
     def _default_val(self, default):
@@ -535,6 +537,20 @@ class Select(Radio):
             return self.default or set()
         return self.default
 
+    @property
+    def option_value_set(self):
+        res = set()
+        for i in self.options:
+            if isinstance(i, tuple):
+                try:
+                    optionValue, _, _ = i
+                except ValueError:
+                    optionValue, _ = i
+            else:
+                optionValue = i
+            res.add(optionValue)
+        return res
+
     def input_html(self, default=None, id_value=None, title=None):
         res = ['<select %stitle="%s" name="%s" %s  size="%d" %s>' % (
             self.id_attr(id_value),
@@ -545,6 +561,12 @@ class Select(Radio):
             " multiple"*(self.multiSelect > 0)
         )]
         default_value = self._default_val(default)
+        if (
+                self.auto_add_option
+                and default_value is not None
+                and default_value not in self.option_value_set
+            ):
+            self.options.insert(0, default_value)
         for i in self.options:
             if isinstance(i, tuple):
                 try:
